/**
 * A JavaScript implementation of the SHA family of hashes - defined in FIPS PUB 180-4, FIPS PUB 202,
 * and SP 800-185 - as well as the corresponding HMAC implementation as defined in FIPS PUB 198-1.
 *
 * Copyright 2008-2022 Brian Turek, 1998-2009 Paul Johnston & Contributors
 * Distributed under the BSD License
 * See http://caligatio.github.com/jsSHA/ for more information
 *
 * Two ECMAScript polyfill functions carry the following license:
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED,
 * INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
 * MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 * See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.
 */
(function(global,factory){typeof exports==="object"&&typeof module!=="undefined"?module.exports=factory():typeof define==="function"&&define.amd?define(factory):(global=typeof globalThis!=="undefined"?globalThis:global||self,global.jsSHA=factory())})(this,function(){"use strict";var extendStatics=function(d,b){extendStatics=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(d,b){d.__proto__=b}||function(d,b){for(var p in b)if(Object.prototype.hasOwnProperty.call(b,p))d[p]=b[p]};return extendStatics(d,b)};function __extends(d,b){if(typeof b!=="function"&&b!==null)throw new TypeError("Class extends value "+String(b)+" is not a constructor or null");extendStatics(d,b);function __(){this.constructor=d}d.prototype=b===null?Object.create(b):(__.prototype=b.prototype,new __)}var b64Tab="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";var arraybuffer_error="ARRAYBUFFER not supported by this environment";var uint8array_error="UINT8ARRAY not supported by this environment";function str2packed(str,utfType,existingPacked,existingPackedLen,bigEndianMod){var codePnt,codePntArr,byteCnt=0,i,j,intOffset,byteOffset,shiftModifier,transposeBytes;existingPackedLen=existingPackedLen||0;var packed=existingPacked||[0],existingByteLen=existingPackedLen>>>3;if("UTF8"===utfType){shiftModifier=bigEndianMod===-1?3:0;for(i=0;i<str.length;i+=1){codePnt=str.charCodeAt(i);codePntArr=[];if(128>codePnt){codePntArr.push(codePnt)}else if(2048>codePnt){codePntArr.push(192|codePnt>>>6);codePntArr.push(128|codePnt&63)}else if(55296>codePnt||57344<=codePnt){codePntArr.push(224|codePnt>>>12,128|codePnt>>>6&63,128|codePnt&63)}else{i+=1;codePnt=65536+((codePnt&1023)<<10|str.charCodeAt(i)&1023);codePntArr.push(240|codePnt>>>18,128|codePnt>>>12&63,128|codePnt>>>6&63,128|codePnt&63)}for(j=0;j<codePntArr.length;j+=1){byteOffset=byteCnt+existingByteLen;intOffset=byteOffset>>>2;while(packed.length<=intOffset){packed.push(0)}packed[intOffset]|=codePntArr[j]<<8*(shiftModifier+bigEndianMod*(byteOffset%4));byteCnt+=1}}}else{shiftModifier=bigEndianMod===-1?2:0;transposeBytes="UTF16LE"===utfType&&bigEndianMod!==1||"UTF16LE"!==utfType&&bigEndianMod===1;for(i=0;i<str.length;i+=1){codePnt=str.charCodeAt(i);if(transposeBytes===true){j=codePnt&255;codePnt=j<<8|codePnt>>>8}byteOffset=byteCnt+existingByteLen;intOffset=byteOffset>>>2;while(packed.length<=intOffset){packed.push(0)}packed[intOffset]|=codePnt<<8*(shiftModifier+bigEndianMod*(byteOffset%4));byteCnt+=2}}return{value:packed,binLen:byteCnt*8+existingPackedLen}}function hex2packed(str,existingPacked,existingPackedLen,bigEndianMod){var i,num,intOffset,byteOffset;if(0!==str.length%2){throw new Error("String of HEX type must be in byte increments")}existingPackedLen=existingPackedLen||0;var packed=existingPacked||[0],existingByteLen=existingPackedLen>>>3,shiftModifier=bigEndianMod===-1?3:0;for(i=0;i<str.length;i+=2){num=parseInt(str.substr(i,2),16);if(!isNaN(num)){byteOffset=(i>>>1)+existingByteLen;intOffset=byteOffset>>>2;while(packed.length<=intOffset){packed.push(0)}packed[intOffset]|=num<<8*(shiftModifier+bigEndianMod*(byteOffset%4))}else{throw new Error("String of HEX type contains invalid characters")}}return{value:packed,binLen:str.length*4+existingPackedLen}}function bytes2packed(str,existingPacked,existingPackedLen,bigEndianMod){var codePnt,i,intOffset,byteOffset;existingPackedLen=existingPackedLen||0;var packed=existingPacked||[0],existingByteLen=existingPackedLen>>>3,shiftModifier=bigEndianMod===-1?3:0;for(i=0;i<str.length;i+=1){codePnt=str.charCodeAt(i);byteOffset=i+existingByteLen;intOffset=byteOffset>>>2;if(packed.length<=intOffset){packed.push(0)}packed[intOffset]|=codePnt<<8*(shiftModifier+bigEndianMod*(byteOffset%4))}return{value:packed,binLen:str.length*8+existingPackedLen}}function b642packed(str,existingPacked,existingPackedLen,bigEndianMod){var byteCnt=0,index,i,j,tmpInt,strPart,intOffset,byteOffset;existingPackedLen=existingPackedLen||0;var packed=existingPacked||[0],existingByteLen=existingPackedLen>>>3,shiftModifier=bigEndianMod===-1?3:0,firstEqual=str.indexOf("=");if(-1===str.search(/^[a-zA-Z0-9=+/]+$/)){throw new Error("Invalid character in base-64 string")}str=str.replace(/=/g,"");if(-1!==firstEqual&&firstEqual<str.length){throw new Error("Invalid '=' found in base-64 string")}for(i=0;i<str.length;i+=4){strPart=str.substr(i,4);tmpInt=0;for(j=0;j<strPart.length;j+=1){index=b64Tab.indexOf(strPart.charAt(j));tmpInt|=index<<18-6*j}for(j=0;j<strPart.length-1;j+=1){byteOffset=byteCnt+existingByteLen;intOffset=byteOffset>>>2;while(packed.length<=intOffset){packed.push(0)}packed[intOffset]|=(tmpInt>>>16-j*8&255)<<8*(shiftModifier+bigEndianMod*(byteOffset%4));byteCnt+=1}}return{value:packed,binLen:byteCnt*8+existingPackedLen}}function uint8array2packed(arr,existingPacked,existingPackedLen,bigEndianMod){var i,intOffset,byteOffset;existingPackedLen=existingPackedLen||0;var packed=existingPacked||[0],existingByteLen=existingPackedLen>>>3,shiftModifier=bigEndianMod===-1?3:0;for(i=0;i<arr.length;i+=1){byteOffset=i+existingByteLen;intOffset=byteOffset>>>2;if(packed.length<=intOffset){packed.push(0)}packed[intOffset]|=arr[i]<<8*(shiftModifier+bigEndianMod*(byteOffset%4))}return{value:packed,binLen:arr.length*8+existingPackedLen}}function arraybuffer2packed(arr,existingPacked,existingPackedLen,bigEndianMod){return uint8array2packed(new Uint8Array(arr),existingPacked,existingPackedLen,bigEndianMod)}function getStrConverter(format,utfType,bigEndianMod){switch(utfType){case"UTF8":case"UTF16BE":case"UTF16LE":break;default:throw new Error("encoding must be UTF8, UTF16BE, or UTF16LE")}switch(format){case"HEX":return function(str,existingBin,existingBinLen){return hex2packed(str,existingBin,existingBinLen,bigEndianMod)};case"TEXT":return function(str,existingBin,existingBinLen){return str2packed(str,utfType,existingBin,existingBinLen,bigEndianMod)};case"B64":return function(str,existingBin,existingBinLen){return b642packed(str,existingBin,existingBinLen,bigEndianMod)};case"BYTES":return function(str,existingBin,existingBinLen){return bytes2packed(str,existingBin,existingBinLen,bigEndianMod)};case"ARRAYBUFFER":try{new ArrayBuffer(0)}catch(ignore){throw new Error(arraybuffer_error)}return function(arr,existingBin,existingBinLen){return arraybuffer2packed(arr,existingBin,existingBinLen,bigEndianMod)};case"UINT8ARRAY":try{new Uint8Array(0)}catch(ignore){throw new Error(uint8array_error)}return function(arr,existingBin,existingBinLen){return uint8array2packed(arr,existingBin,existingBinLen,bigEndianMod)};default:throw new Error("format must be HEX, TEXT, B64, BYTES, ARRAYBUFFER, or UINT8ARRAY")}}function packed2hex(packed,outputLength,bigEndianMod,formatOpts){var hex_tab="0123456789abcdef";var str="",i,srcByte;var length=outputLength/8,shiftModifier=bigEndianMod===-1?3:0;for(i=0;i<length;i+=1){srcByte=packed[i>>>2]>>>8*(shiftModifier+bigEndianMod*(i%4));str+=hex_tab.charAt(srcByte>>>4&15)+hex_tab.charAt(srcByte&15)}return formatOpts["outputUpper"]?str.toUpperCase():str}function packed2b64(packed,outputLength,bigEndianMod,formatOpts){var str="",i,j,triplet,int1,int2;var length=outputLength/8,shiftModifier=bigEndianMod===-1?3:0;for(i=0;i<length;i+=3){int1=i+1<length?packed[i+1>>>2]:0;int2=i+2<length?packed[i+2>>>2]:0;triplet=(packed[i>>>2]>>>8*(shiftModifier+bigEndianMod*(i%4))&255)<<16|(int1>>>8*(shiftModifier+bigEndianMod*((i+1)%4))&255)<<8|int2>>>8*(shiftModifier+bigEndianMod*((i+2)%4))&255;for(j=0;j<4;j+=1){if(i*8+j*6<=outputLength){str+=b64Tab.charAt(triplet>>>6*(3-j)&63)}else{str+=formatOpts["b64Pad"]}}}return str}function packed2bytes(packed,outputLength,bigEndianMod){var str="",i,srcByte;var length=outputLength/8,shiftModifier=bigEndianMod===-1?3:0;for(i=0;i<length;i+=1){srcByte=packed[i>>>2]>>>8*(shiftModifier+bigEndianMod*(i%4))&255;str+=String.fromCharCode(srcByte)}return str}function packed2arraybuffer(packed,outputLength,bigEndianMod){var i;var length=outputLength/8,retVal=new ArrayBuffer(length),arrView=new Uint8Array(retVal),shiftModifier=bigEndianMod===-1?3:0;for(i=0;i<length;i+=1){arrView[i]=packed[i>>>2]>>>8*(shiftModifier+bigEndianMod*(i%4))&255}return retVal}function packed2uint8array(packed,outputLength,bigEndianMod){var i;var length=outputLength/8,shiftModifier=bigEndianMod===-1?3:0,retVal=new Uint8Array(length);for(i=0;i<length;i+=1){retVal[i]=packed[i>>>2]>>>8*(shiftModifier+bigEndianMod*(i%4))&255}return retVal}function getOutputConverter(format,outputBinLen,bigEndianMod,outputOptions){switch(format){case"HEX":return function(binarray){return packed2hex(binarray,outputBinLen,bigEndianMod,outputOptions)};case"B64":return function(binarray){return packed2b64(binarray,outputBinLen,bigEndianMod,outputOptions)};case"BYTES":return function(binarray){return packed2bytes(binarray,outputBinLen,bigEndianMod)};case"ARRAYBUFFER":try{new ArrayBuffer(0)}catch(ignore){throw new Error(arraybuffer_error)}return function(binarray){return packed2arraybuffer(binarray,outputBinLen,bigEndianMod)};case"UINT8ARRAY":try{new Uint8Array(0)}catch(ignore){throw new Error(uint8array_error)}return function(binarray){return packed2uint8array(binarray,outputBinLen,bigEndianMod)};default:throw new Error("format must be HEX, B64, BYTES, ARRAYBUFFER, or UINT8ARRAY")}}var TWO_PWR_32=4294967296;var sha_variant_error="Chosen SHA variant is not supported";var mac_rounds_error="Cannot set numRounds with MAC";function getOutputOpts(options){var retVal={outputUpper:false,b64Pad:"=",outputLen:-1},outputOptions=options||{},lenErrstr="Output length must be a multiple of 8";retVal["outputUpper"]=outputOptions["outputUpper"]||false;if(outputOptions["b64Pad"]){retVal["b64Pad"]=outputOptions["b64Pad"]}if(outputOptions["outputLen"]){if(outputOptions["outputLen"]%8!==0){throw new Error(lenErrstr)}retVal["outputLen"]=outputOptions["outputLen"]}else if(outputOptions["shakeLen"]){if(outputOptions["shakeLen"]%8!==0){throw new Error(lenErrstr)}retVal["outputLen"]=outputOptions["shakeLen"]}if("boolean"!==typeof retVal["outputUpper"]){throw new Error("Invalid outputUpper formatting option")}if("string"!==typeof retVal["b64Pad"]){throw new Error("Invalid b64Pad formatting option")}return retVal}function parseInputOption(key,value,bigEndianMod,fallback){var errStr=key+" must include a value and format";if(!value){if(!fallback){throw new Error(errStr)}return fallback}if(typeof value["value"]==="undefined"||!value["format"]){throw new Error(errStr)}return getStrConverter(value["format"],value["encoding"]||"UTF8",bigEndianMod)(value["value"])}var jsSHABase=function(){function jsSHABase(variant,inputFormat,options){var inputOptions=options||{};this.inputFormat=inputFormat;this.utfType=inputOptions["encoding"]||"UTF8";this.numRounds=inputOptions["numRounds"]||1;if(isNaN(this.numRounds)||this.numRounds!==parseInt(this.numRounds,10)||1>this.numRounds){throw new Error("numRounds must a integer >= 1")}this.shaVariant=variant;this.remainder=[];this.remainderLen=0;this.updateCalled=false;this.processedLen=0;this.macKeySet=false;this.keyWithIPad=[];this.keyWithOPad=[]}jsSHABase.prototype.update=function(srcString){var i,updateProcessedLen=0;var variantBlockIntInc=this.variantBlockSize>>>5,convertRet=this.converterFunc(srcString,this.remainder,this.remainderLen),chunkBinLen=convertRet["binLen"],chunk=convertRet["value"],chunkIntLen=chunkBinLen>>>5;for(i=0;i<chunkIntLen;i+=variantBlockIntInc){if(updateProcessedLen+this.variantBlockSize<=chunkBinLen){this.intermediateState=this.roundFunc(chunk.slice(i,i+variantBlockIntInc),this.intermediateState);updateProcessedLen+=this.variantBlockSize}}this.processedLen+=updateProcessedLen;this.remainder=chunk.slice(updateProcessedLen>>>5);this.remainderLen=chunkBinLen%this.variantBlockSize;this.updateCalled=true;return this};jsSHABase.prototype.getHash=function(format,options){var i,finalizedState,outputBinLen=this.outputBinLen;var outputOptions=getOutputOpts(options);if(this.isVariableLen){if(outputOptions["outputLen"]===-1){throw new Error("Output length must be specified in options")}outputBinLen=outputOptions["outputLen"]}var formatFunc=getOutputConverter(format,outputBinLen,this.bigEndianMod,outputOptions);if(this.macKeySet&&this.getMAC){return formatFunc(this.getMAC(outputOptions))}finalizedState=this.finalizeFunc(this.remainder.slice(),this.remainderLen,this.processedLen,this.stateCloneFunc(this.intermediateState),outputBinLen);for(i=1;i<this.numRounds;i+=1){if(this.isVariableLen&&outputBinLen%32!==0){finalizedState[finalizedState.length-1]&=16777215>>>24-outputBinLen%32}finalizedState=this.finalizeFunc(finalizedState,outputBinLen,0,this.newStateFunc(this.shaVariant),outputBinLen)}return formatFunc(finalizedState)};jsSHABase.prototype.setHMACKey=function(key,inputFormat,options){if(!this.HMACSupported){throw new Error("Variant does not support HMAC")}if(this.updateCalled){throw new Error("Cannot set MAC key after calling update")}var keyOptions=options||{},keyConverterFunc=getStrConverter(inputFormat,keyOptions["encoding"]||"UTF8",this.bigEndianMod);this._setHMACKey(keyConverterFunc(key))};jsSHABase.prototype._setHMACKey=function(key){var blockByteSize=this.variantBlockSize>>>3,lastArrayIndex=blockByteSize/4-1;var i;if(this.numRounds!==1){throw new Error(mac_rounds_error)}if(this.macKeySet){throw new Error("MAC key already set")}if(blockByteSize<key["binLen"]/8){key["value"]=this.finalizeFunc(key["value"],key["binLen"],0,this.newStateFunc(this.shaVariant),this.outputBinLen)}while(key["value"].length<=lastArrayIndex){key["value"].push(0)}for(i=0;i<=lastArrayIndex;i+=1){this.keyWithIPad[i]=key["value"][i]^909522486;this.keyWithOPad[i]=key["value"][i]^1549556828}this.intermediateState=this.roundFunc(this.keyWithIPad,this.intermediateState);this.processedLen=this.variantBlockSize;this.macKeySet=true};jsSHABase.prototype.getHMAC=function(format,options){var outputOptions=getOutputOpts(options),formatFunc=getOutputConverter(format,this.outputBinLen,this.bigEndianMod,outputOptions);return formatFunc(this._getHMAC())};jsSHABase.prototype._getHMAC=function(){var finalizedState;if(!this.macKeySet){throw new Error("Cannot call getHMAC without first setting MAC key")}var firstHash=this.finalizeFunc(this.remainder.slice(),this.remainderLen,this.processedLen,this.stateCloneFunc(this.intermediateState),this.outputBinLen);finalizedState=this.roundFunc(this.keyWithOPad,this.newStateFunc(this.shaVariant));finalizedState=this.finalizeFunc(firstHash,this.outputBinLen,this.variantBlockSize,finalizedState,this.outputBinLen);return finalizedState};return jsSHABase}();function rotl_32(x,n){return x<<n|x>>>32-n}function parity_32(x,y,z){return x^y^z}function ch_32(x,y,z){return x&y^~x&z}function maj_32(x,y,z){return x&y^x&z^y&z}function safeAdd_32_2(a,b){var lsw=(a&65535)+(b&65535),msw=(a>>>16)+(b>>>16)+(lsw>>>16);return(msw&65535)<<16|lsw&65535}function safeAdd_32_5(a,b,c,d,e){var lsw=(a&65535)+(b&65535)+(c&65535)+(d&65535)+(e&65535),msw=(a>>>16)+(b>>>16)+(c>>>16)+(d>>>16)+(e>>>16)+(lsw>>>16);return(msw&65535)<<16|lsw&65535}function getNewState(_variant){return[1732584193,4023233417,2562383102,271733878,3285377520]}function roundSHA1(block,H){var a,b,c,d,e,T,t;var W=[];a=H[0];b=H[1];c=H[2];d=H[3];e=H[4];for(t=0;t<80;t+=1){if(t<16){W[t]=block[t]}else{W[t]=rotl_32(W[t-3]^W[t-8]^W[t-14]^W[t-16],1)}if(t<20){T=safeAdd_32_5(rotl_32(a,5),ch_32(b,c,d),e,1518500249,W[t])}else if(t<40){T=safeAdd_32_5(rotl_32(a,5),parity_32(b,c,d),e,1859775393,W[t])}else if(t<60){T=safeAdd_32_5(rotl_32(a,5),maj_32(b,c,d),e,2400959708,W[t])}else{T=safeAdd_32_5(rotl_32(a,5),parity_32(b,c,d),e,3395469782,W[t])}e=d;d=c;c=rotl_32(b,30);b=a;a=T}H[0]=safeAdd_32_2(a,H[0]);H[1]=safeAdd_32_2(b,H[1]);H[2]=safeAdd_32_2(c,H[2]);H[3]=safeAdd_32_2(d,H[3]);H[4]=safeAdd_32_2(e,H[4]);return H}function finalizeSHA1(remainder,remainderBinLen,processedBinLen,H){var i;var offset=(remainderBinLen+65>>>9<<4)+15,totalLen=remainderBinLen+processedBinLen;while(remainder.length<=offset){remainder.push(0)}remainder[remainderBinLen>>>5]|=128<<24-remainderBinLen%32;remainder[offset]=totalLen&4294967295;remainder[offset-1]=totalLen/TWO_PWR_32|0;for(i=0;i<remainder.length;i+=16){H=roundSHA1(remainder.slice(i,i+16),H)}return H}var jsSHA=function(_super){__extends(jsSHA,_super);function jsSHA(variant,inputFormat,options){var _this=this;if("SHA-1"!==variant){throw new Error(sha_variant_error)}_this=_super.call(this,variant,inputFormat,options)||this;var resolvedOptions=options||{};_this.HMACSupported=true;_this.getMAC=_this._getHMAC;_this.bigEndianMod=-1;_this.converterFunc=getStrConverter(_this.inputFormat,_this.utfType,_this.bigEndianMod);_this.roundFunc=roundSHA1;_this.stateCloneFunc=function(state){return state.slice()};_this.newStateFunc=getNewState;_this.finalizeFunc=finalizeSHA1;_this.intermediateState=getNewState();_this.variantBlockSize=512;_this.outputBinLen=160;_this.isVariableLen=false;if(resolvedOptions["hmacKey"]){_this._setHMACKey(parseInputOption("hmacKey",resolvedOptions["hmacKey"],_this.bigEndianMod))}return _this}return jsSHA}(jsSHABase);return jsSHA});
